# -*- coding: utf-8 -*-

"""Top-level package for safescope."""
from .safescope import assert_local_vars, get_non_local_var_names

__author__ = """Haavard Kvamme"""
__email__ = 'haavard.kvamme@gmail.com'
__version__ = '0.1.8'
